@@ -273,11 +273,17 @@ end_options:
 		  unsigned i;
 
 		  length = fieldlen ? fieldlen : format_size_in_decimal(n);
-		  for (i = 0; i < length; i++)
+		  if (n) 
 		    {
-		      buffer[length - i - 1] = n ? '0' + n % 10 : zeropad ? '0' : ' ';
-		      n /= 10;
+		      for (i = 0; i < length; i++)
+			{
+			  buffer[length - i - 1] = n ? '0' + n % 10 : zeropad ? '0' : ' ';
+			  n /= 10;
+			}
 		    }
+		  else
+		    buffer[0] = '0';
+
 		}
 		else {
 		  char hexchars[] = "0123456789abcdef";
