@@ -784,6 +784,10 @@ static int inet_address2sockaddr_in(struct address_info *c, int sasize, struct s
 
 	if (sasize != sizeof(*in))
 		return 0;
+	if (a->sa.sin_addr.s_addr != 0) {
+		memcpy(sa, &a->sa, sizeof(a->sa));
+		return 1;
+	}
 	if (a->ip && !tcp_addr(in,
 				a->ip->length, a->ip->data,
 				a->port))
@@ -932,16 +936,19 @@ struct address_info *sockaddr2address_info(size_t addr_len,
 	{
 	case AF_INET:
 	{
+		struct inet_address_info *a;
 		struct sockaddr_in *in = (struct sockaddr_in *) addr;
 		UINT32 ip, port;
 
 		ip = ntohl(in->sin_addr.s_addr);
 		port = ntohs(in->sin_port);
-		return &make_inet_address(c_format("%i.%i.%i.%i",
-						  (ip >> 24) & 0xff,
-						  (ip >> 16) & 0xff,
-						  (ip >> 8) & 0xff,
-						  ip & 0xff), port)->super;
+		a = make_inet_address(c_format("%i.%i.%i.%i%c",
+					       (ip >> 24) & 0xff,
+					       (ip >> 16) & 0xff,
+					       (ip >> 8) & 0xff,
+					       ip & 0xff, 0), port);
+		memcpy(&a->sa, addr, addr_len);
+		return &a->super;
 	}
 	case AF_UNIX: {
 		struct sockaddr_un *un = (struct sockaddr_un *) addr;
