@@ -27,6 +27,15 @@
 
 #include <assert.h>
 
+static void do_queue_mark(struct ol_queue *q,
+                                 void (*mark)(struct ol_object *o));
+
+static void do_queue_free(struct ol_queue *q);
+
+#define CLASS_DEFINE
+#include "queue.h.x"
+#undef CLASS_DEFINE
+
 /* Short cuts */
 #define next np_links[LSH_QUEUE_NEXT]
 #define prev np_links[LSH_QUEUE_PREV]
@@ -147,3 +156,105 @@ struct ol_queue_node *ol_queue_remove_tail(struct ol_queue *q)
 }
 
   
+/* object_queue */
+static struct object_queue_node *
+make_object_queue_node(struct ol_object *o)
+{
+  struct object_queue_node *n;
+
+  NEW_SPACE(n);
+  n->o = o;
+
+  return n;
+}
+
+struct object_queue *make_object_queue(void)
+{
+  NEW(object_queue, q);
+  ol_queue_init(&q->q);
+  return q;
+}
+
+int object_queue_is_empty(struct object_queue *q)
+{
+  return EMPTYP(&q->q);
+}
+
+struct object_queue_node *
+object_queue_add_head(struct object_queue *q, struct ol_object *o)
+{
+  struct object_queue_node *n = make_object_queue_node(o);
+
+  ol_queue_add_head(&q->q, &n->header);
+  return n;
+}
+
+struct object_queue_node *
+object_queue_add_tail(struct object_queue *q, struct ol_object *o)
+{
+  struct object_queue_node *n = make_object_queue_node(o);
+  ol_queue_add_tail(&q->q, &n->header);
+  return n;
+}
+
+static struct ol_object *
+object_queue_get_contents(struct ol_queue_node *l)
+{
+  struct object_queue_node *n = (struct object_queue_node *) l;
+
+  struct ol_object *res = n->o;
+  ol_space_free(n);
+
+  return res;
+}
+
+static struct ol_object *
+object_queue_peek(struct ol_queue_node *n)
+{
+  return ( (struct object_queue_node *) n)->o;
+}
+
+void object_queue_remove(struct object_queue_node *n)
+{
+  ol_queue_remove(&n->header);
+  ol_space_free(n);
+}
+
+struct ol_object *object_queue_remove_head(struct object_queue *q)
+{
+  return object_queue_get_contents(ol_queue_remove_head(&q->q));
+}
+
+struct ol_object *object_queue_remove_tail(struct object_queue *q)
+{
+  return object_queue_get_contents(ol_queue_remove_tail(&q->q));
+}
+
+struct ol_object *object_queue_peek_head(struct object_queue *q)
+{
+  return EMPTYP(&q->q) ? NULL : object_queue_peek(q->q.head);
+}
+
+struct ol_object *object_queue_peek_tail(struct object_queue *q)
+{
+  return EMPTYP(&q->q) ? NULL : object_queue_peek(q->q.tailprev);
+}
+
+/* For gc */
+static void do_queue_mark(struct ol_queue *q,
+                                 void (*mark)(struct ol_object *o))
+{
+  FOR_QUEUE(q, struct object_queue_node *, n)
+    mark(n->o);
+}
+
+static void do_queue_free(struct ol_queue *q)
+{
+  FOR_QUEUE(q, struct object_queue_node *, n)
+    ol_space_free(n);
+}
+
+void object_queue_kill(struct object_queue *q)
+{
+  do_queue_free(&q->q);
+}
