@@ -45,7 +45,7 @@
 struct malloc_entry *first_malloc = NULL;
 
 #define SIZE_IN_INTS(x) (((x) + sizeof(int)-1) / sizeof(int))
-/* #define REALLY_DEBUG_ALLOC 1 */
+#define REALLY_DEBUG_ALLOC 1
 
 #if REALLY_DEBUG_ALLOC
 UINT32 strcount = 0;
